@@ -103,31 +103,7 @@ suffixes = ["html", "xhtml"]
 	b.Assert(contentTypes.Markdown.Suffixes(), qt.DeepEquals, []string{"md", "mdown", "markdown"})
 }
 
-func TestPaginationConfigOld(t *testing.T) {
-	files := `
--- hugo.toml --
- [languages.en]
- weight = 1
- paginatePath = "page-en"
- 
- [languages.de]
- weight = 2
- paginatePath = "page-de"
- paginate = 20
-`
-
-	b := hugolib.Test(t, files)
-
-	confEn := b.H.Sites[0].Conf.Pagination()
-	confDe := b.H.Sites[1].Conf.Pagination()
-
-	b.Assert(confEn.Path, qt.Equals, "page-en")
-	b.Assert(confEn.PagerSize, qt.Equals, 10)
-	b.Assert(confDe.Path, qt.Equals, "page-de")
-	b.Assert(confDe.PagerSize, qt.Equals, 20)
-}
-
-func TestPaginationConfigNew(t *testing.T) {
+func TestPaginationConfig(t *testing.T) {
 	files := `
 -- hugo.toml --
  [languages.en]
@@ -184,3 +160,18 @@ title: "p3"
 	b.AssertFileExists("public/page/1/index.html", false)
 	b.AssertFileContent("public/page/2/index.html", "pagination-default")
 }
+
+func TestMapUglyURLs(t *testing.T) {
+	files := `
+-- hugo.toml --
+[uglyurls]
+  posts = true
+`
+
+	b := hugolib.Test(t, files)
+
+	c := b.H.Configs.Base
+
+	b.Assert(c.C.IsUglyURLSection("posts"), qt.IsTrue)
+	b.Assert(c.C.IsUglyURLSection("blog"), qt.IsFalse)
+}
